"""
    Module to run scripts from Telegrma by calling custom commands
    For instant, good_morning will call the command defined in
    /script good_morning will call the command defined in [SCRIPTS] good_morning
"""
import os
import logging
import subprocess as sp
from pybliotecario.components.component_core import Component

log = logging.getLogger(__name__)


class Script(Component):
    section_name = "SCRIPT"

    def __init__(self, telegram_object, **kwargs):
        super().__init__(telegram_object, **kwargs)
        self.blocked = False
        self.scripts = self.read_config_section()
        if not self.scripts:
            self.blocked = True
        default_keys = set(self.configuration["DEFAULT"].keys())
        self.script_names = list(set(self.scripts.keys()) - default_keys)

    def available_commands(self):
        """ Sends a list with the available commands """
        msg_str = "Available commands: " + ", ".join(self.script_names)
        self.send_msg(msg_str)

    @classmethod
    def configure_me(cls):
        print("")
        print(" # Scripts module ")
        print("This is the configuration helper for the scripts module, leave empty to exit")
        dict_out = {cls.section_name: {}}
        while True:
            script_command = input(" Introduce the command name: ")
            sc_cmd = script_command.strip()
            if not sc_cmd:
                break
            script_file = input(" Introduce the path of the command to run with '{0}': ".format(sc_cmd))
            dict_out[cls.section_name][sc_cmd] = script_file
        return dict_out

    def telegram_message(self, msg):
        if not self.check_identity(msg):
            self.blocked = True
            self.send_msg("You are not allowed to run scripts here")
        if self.blocked:
            return
        command_name = msg.text.strip()
        if not msg.has_arguments:
            self.send_msg("Add a command name after /script")
        elif command_name == "list":
            self.available_commands()
        elif command_name not in self.script_names:
            self.send_msg("Command {0} not defined".format(command_name))
            self.available_commands()
        else:
            command_path = self.scripts[command_name]
            log.info("Running: {0}, path: {1}".format(command_name, command_path))
            if os.path.isfile(command_path):
                sp.run([command_path])
                self.send_msg("Command ran")
            else:
                self.send_msg("Script not found")



"""
    This module contains a mapping between Telegram commands
    (msgs which start with / ) and components which will act on them.

    Note that the components are only imported when/if the appropiate command is invoked
    This is a design choice as this way it is not necessary to have all dependencies
    if you want to run only some submodules of the pybliotecario.
"""

import subprocess as sp
import os
import logging

log = logging.getLogger(__name__)


def act_on_telegram_command(tele_api, message_obj, config):
    """
    Act for a given telegram command
    """
    tg_command = message_obj.command
    chat_id = config["DEFAULT"]["chat_id"]

    if tg_command == "ip":
        from pybliotecario.components.ip_lookup import IpLookup as Actor
    elif tg_command.lower() in ("is_pid_alive", "kill_pid"):
        from pybliotecario.components.pid import ControllerPID as Actor
    elif tg_command in ("arxiv-query", "arxiv", "arxivget", "arxiv-get"):
        from pybliotecario.components.arxiv_mod import Arxiv as Actor
    elif tg_command == "script":
        from pybliotecario.components.scripts import Script as Actor
    elif tg_command in ("r", "roll"):
        from pybliotecario.components.dnd import DnD as Actor
    else:
        log.info("No actor declared for this command: {0}".format(tg_command))
        return None



    actor_instance = Actor(
        tele_api, chat_id=chat_id, configuration=config, interaction_chat=message_obj.chat_id, running_in_loop=True
    )
    return actor_instance.telegram_message(message_obj)



"""
    This module contains utilities for interacting with processes of the computer
    in which the bot runs.
    It allows for things like killing a process from Telegram or querying for
    an active process
"""

import psutil
from pybliotecario.components.component_core import Component

import logging

log = logging.getLogger(__name__)


def get_process(pid):
    """ Returns a process object for the given PID """
    exists = psutil.pid_exists(pid)
    proc = None
    if exists:
        try:
            proc = psutil.Process(pid=pid)
        except psutil.NoSuchProcess:
            pass

    if proc is None:
        log.info("WARNING: Process {0} was not found".format(pid))

    return proc


def wait_for_it_until_finished(pids):
    """ Receives a list of PIDs and wait for them """
    processes = [get_process(i) for i in pids]
    psutil.wait_procs(processes)


def is_it_alive(data):
    """ Given a pid or a string, check whether
    there is any matching process alive """
    alive = False
    matches = []
    if data.isdigit():
        alive = psutil.pid_exists(int(data))
    else:
        all_active_processes = psutil.process_iter()
        for proc in all_active_processes:
            cmdline = proc.cmdline()
            for info in cmdline:
                if data in info:
                    matches.append("PID: {0}, {1}".format(proc.pid, " ".join(cmdline)))
                    alive = True
    if alive:
        msg = "{0} is alive".format(data)
        if matches:
            msg += "\nI found the following matching processes: \n > {0}".format("\n > ".join(matches))

    else:
        msg = "{0} not found among active processes".format(data)
    return msg


def kill_pid(pid):
    """ Kills the given pid """
    process = get_process(pid)
    if process is not None:
        process.kill()
        return "{0} killed".format(pid)
    else:
        return "No process with pid {0}".format(pid)


class ControllerPID(Component):
    """
    """

    def cmdline_command(self, args):
        """ Waits until the given PID(s) are finished """
        log.info("Waiting for the given PIDs: {0}".format(args.pid))
        wait_for_it_until_finished(args.pid)

    @staticmethod
    def kill(pid):
        """ Kills the received PID """
        return kill_pid(pid)

    @staticmethod
    def alive(pid):
        """ Check whether a PID (or str for searching
        for a PID) is alive """
        return is_it_alive(pid)

    def telegram_message(self, msg):
        if self.check_identity(msg):
            pid_string = msg.text.strip()
            if msg.command == "kill_pid":
                if pid_string.isdigit:
                    return_msg = self.kill(int(pid_string))
                else:
                    return_msg = "{0} is not a PID?".format(pid_string)
            elif msg.command == "is_pid_alive":
                return_msg = self.alive(pid_string)
        else:
            return_msg = "You are not allowed to use this"
        self.send_msg(return_msg)



#!/usr/bin/env python3
import json
import os.path
import urllib
import requests

TELEGRAM_URL = "https://api.telegram.org/"
import logging

log = logging.getLogger(__name__)

def log_request(status_code, reason, content):
    """ Log the status of the send requests """
    result = "Request sent, status code: {0} - {1}: {2}".format(status_code, reason, content)
    log.info(result)

class TelegramUtil:
    """ This class handles all comunications with
    Telegram """

    def __init__(self, TOKEN, debug=False, timeout=300):
        self.offset = None
        self.debug = debug
        self.timeout = timeout
        # Build app the API urls
        base_URL = TELEGRAM_URL + "bot{}/".format(TOKEN)
        self.base_fileURL = TELEGRAM_URL + "file/bot{}/".format(TOKEN)
        self.send_msg = base_URL + "sendMessage"
        self.send_img = base_URL + "sendPhoto"
        self.send_doc = base_URL + "sendDocument"
        self.get_msg = base_URL + "getUpdates"
        self.get_file = base_URL + "getFile"

    def __make_request(self, url):
        """ Returns the response for a given url
        In case of timeout, emulate an empty response
        """
        try:
            response = requests.get(url, timeout=self.timeout)
            content = response.content.decode("utf-8")
        except requests.exceptions.Timeout:
            content = '{ "ok": true, "result": [] }'
        return content

    def __get_json_from_url(self, url):
        """ Return the json response of a given url """
        content_response = self.__make_request(url)
        return json.loads(content_response)

    def __re_offset(self, updates):
        """ Updates the offset
        i.e., resets don't ask Telegram server for old messages
        """
        if not updates:  # len(updates) == 0
            return
        li = []
        for update in updates:
            li.append(int(update["update_id"]))
        self.offset = max(li) + 1

    def get_filePath(self, fileId):
        """ Given a file id, retrieve the URI of the file
        in the remote server
        """
        url = self.get_file + "?file_id={0}".format(fileId)
        json = self.__get_json_from_url(url)
        # was it ok?
        if json["ok"]:
            fpath = json["result"]["file_path"]
            return self.base_fileURL + fpath
        else:
            log.info(json["error_code"])
            log.info("Here's all the information we have on this request")
            log.info("This is the url we have used")
            log.info(url)
            return None

    def get_updates(self, not_empty=False):
        """
        Returns a json with the last messages the bot has received
        when an offset is found, previous msg are not retrieved
        We use longpolling to keep the connection open for timeout seconds

        If not_empty = True, this function will only return when a message arrives
        """
        url = "{0}?timeout={1}".format(self.get_msg, self.timeout)
        if self.offset:
            url += "&offset={0}".format(self.offset)
        updates = self.__get_json_from_url(url)
        if not updates and not_empty:
            return self.get_updates(not_empty=True)
        if self.debug:
            log.info("Request url: {0}".format(url))
            log.info("Obtained updates: {0}".format(updates))
        try:
            result = updates["result"]
        except Exception as e:
            # in case of ANY exception, just log.info it out and let the program run
            log.info("Error: ")
            log.info(str(e))
            log.info("List of updates: ")
            log.info(updates)
            return []
        self.__re_offset(result)
        return result

    def send_message(self, text, chat):
        """ Send a message to a given chat """
        text = urllib.parse.quote_plus(text)
        url = self.send_msg + "?text={}&chat_id={}".format(text, chat)
        self.__make_request(url)

    def send_image(self, imgPath, chat):
        """ Send an image to a given chat """
        data = {"chat_id": chat}
        img = open(imgPath, "rb")
        files = {"photo": ("picture.jpg", img)}  # Here, the ,"rb" thing
        blabla = requests.post(self.send_img, data=data, files=files)
        log_request(blabla.status_code, blabla.reason, blabla.content)

    def send_file(self, filePath, chat):
        data = {"chat_id": chat}
        file_stream = open(filePath, "rb")
        doc_name = os.path.basename(filePath)
        files = {"document": (doc_name, file_stream)}
        blabla = requests.post(self.send_doc, data=data, files=files)
        log_request(blabla.status_code, blabla.reason, blabla.content)

    def send_file_by_url(self, file_url, chat):
        """
        Sends a file using the telegram api which allows to send by url
        it only works for images/pdf, and not all pdfs
        """
        data = {"chat_id": chat, "document": file_url}
        blabla = requests.post(self.send_doc, data=data)
        log.info(blabla.status_code, blabla.reason, blabla.content)

    def download_file(self, fileId, file_name_raw):
        """ Download file defined by fileId
        to given file_name """
        file_url = self.get_filePath(fileId)
        if not file_url:
            return None
        file_name = file_name_raw
        n = 0
        while os.path.isfile(file_name):
            filedir = os.path.dirname(file_name_raw)
            basename = os.path.basename(file_name_raw)
            file_name = "{0}/n{1}-{2}".format(filedir, n, basename)
            n += 1
        return urllib.request.urlretrieve(file_url, file_name)


if __name__ == "__main__":
    log.info("Testing TelegramUtil")
    TOKEN = "must put a token here to test"
    ut = TelegramUtil(TOKEN, debug=True)
    results = ut.get_updates()
    for result in results:
        log.info("Complete json:")
        log.info(result)
        message = result["message"]
        chat_id = message["chat"]["id"]
        txt = message["text"]
        log.info("Message from {0}: {1}".format(chat_id, txt))
        ut.send_message("Message received", chat_id)
    ut.timeout = 1
    ut.get_updates()  # Use the offset to confirm updates



"""
    All different components must inherit from the Component class

    If the component `configurable` flag is set to True,
    the installer will call the `configure_me` method of the class.

    Once a command has been registered to a component it will call the
    `telegram_command_parser` or the `cmdline_command_parser`.

    In the simplest scenario (the one considered by this example class)
    the class Component will just pass the text of the msg (or the command)
    to the `act_on_command` or `act_on_message` methods.
"""
import os
import sys
import logging

from pybliotecario.argument_parser import CONFIG_FILE

log = logging.getLogger(__name__)


class Component:
    """
    This is the base class from which all components should inherit

    Instantiate the class requires a messeger object able to talk with the telegram API
    optionally it can receive a configuration object and a chat_id object.
    In general chat_id will be contained already in the configuration
    however we want to be able to comunicate with chats not included in the comunication,
    that's why it is left as a separate option
    """

    def __init__(self, telegram_object, chat_id=None, configuration=None, interaction_chat=None, running_in_loop=False):
        self.telegram = telegram_object
        self.chat_id = chat_id
        if interaction_chat is None:
            self.interaction_chat = chat_id
        else:
            self.interaction_chat = interaction_chat
        self.configuration = configuration
        self.configurable = False
        self.running_in_loop = running_in_loop

    def update_config(self):
        """ Updates default ($HOME/.CONFIG_FILE) configuration file """
        default_config = "{0}/.{1}".format(os.environ.get("HOME"), CONFIG_FILE)
        new_section = self.configure_me()
        for key, item in new_section.items():
            self.configuration[key] = item
        with open(default_config, "w") as f:
            self.configuration.write(f)

    def read_config_section(self, section=None):
        """
        Checks whether section exists within the configuration file
        returns None if it doesn't

        If the bot is running in daemon/receiving from Telegram mode, can't configure
        """
        if section is None:
            section = self.section_name
        try:
            section_dict = self.configuration[section]
            return section_dict
        except KeyError:
            if self.running_in_loop:
                self.send_msg("Section {0} is not configured and will not work".format(section))
                log.error("Section {0} is not configured, please run --init".format(section))
                return {}
            log.warning("There is no section {0} in configuration file".format(section))
            yesno = input(
                "Do you want to configure? (this will add a new section to $HOME/.{0} [yn] ".format(CONFIG_FILE)
            )
            if yesno.lower() in ("y", "s"):
                self.update_config()
                sys.exit(-1)
            else:
                log.error("Exiting with error")
                sys.exit(-1)

    @classmethod
    def whoamI(cls):
        return cls.__name__

    @classmethod
    def configure_me(cls):
        """ In first initialization (--init or --config) this method will be called
        if any configuration is needed for the child class, it should be done here """
        return None

    @staticmethod
    def split_list(comma_separated_str):
        """ Receives a string representation of a comma separated list
        and splits it, filters out empty spaces and returns a list """
        list_str = comma_separated_str.strip().split(",")
        filtered_str = filter(lambda x: x, list_str)
        return list(filtered_str)

    def check_identity(self, msg):
        """ Checks that the user asking is the correct one """
        if int(self.chat_id) == int(msg.chat_id):
            return True
        else:
            return False

    def telegram_message(self, msg):
        """ Recevies a `msg` object and then
        decides what to do with it.
        The `msg` object includes the Telegram command"""
        self.act_on_message(msg.text.strip())

    def cmdline_command(self, args):
        """
        Receives the full set of arguments (including, of course,
        the one which invoked the class) in case synergies with
        other command line arguments are needed.
        In the most general case, of course, this will just be ignored
        """
        self.act_on_command()

    def act_on_message(self, content=None):
        """ Acts on a received msg """
        self.telegram.send_message("Msg received", self.chat_id)

    def act_on_command(self, content=None):
        """ Acts on a received command """
        self.telegram.send_message("Comand line argument invoked", self.chat_id)

    # Some useful wrappers
    def send_msg(self, msg, chat_id=None):
        """ Wrapper around API send_msg, if chat_id is not defined
        it will use the chat_id this class was instantiated to """
        if chat_id is None:
            chat_id = self.interaction_chat
        return self.telegram.send_message(msg, chat_id)

    def send_file(self, filepath, chat_id=None, delete=False):
        """ Wrapper around API send_file, if chat_id is not defined
        it will use the chat_id this class was instantiated to.
        It will check for existence of the file and will delete it afterwards if required
        """
        if chat_id is None:
            chat_id = self.interaction_chat
        if not os.path.isfile(filepath):
            self.send_msg("ERROR: failed to send {0}".format(filepath), chat_id)
        self.telegram.send_file(filepath, chat_id)
        if delete:
            os.remove(filepath)



"""
    Module containing functions to deal with Git and Mercurial repositories.
    Useful for monitoring changes in a repository being able to apply custom filters
"""

import os
import re
import subprocess as sp
from pybliotecario.components.component_core import Component
import logging

log = logging.getLogger(__name__)

re_hg_branch = re.compile("(?<=branch:).*(?=\n)")
re_hg_user = re.compile("(?<=user:).*(?=<)")
re_hg_msg = re.compile("(?<=summary:).*(?=\n)")

re_git_user = re.compile("(?<=From:).*(?=<)")
re_git_msg = re.compile("(?<=Subject:).*(?=\n\n)")


def mercurial_incoming():
    """ Performs mercurial incoming and prettifies it """
    cmd = ["hg", "incoming"]
    cmd_ran = sp.run(cmd, stdout=sp.PIPE)
    out = cmd_ran.stdout
    changesets = out.decode().split("changeset")[1:]
    rev_dicts = []
    for rev in changesets:
        branch = re_hg_branch.search(rev).group().strip()
        user = re_hg_user.search(rev).group().strip()
        msg = re_hg_msg.search(rev).group().strip()
        rev_dicts.append({"branch": branch, "user": user, "msg": msg})
    return rev_dicts


def mercurial_pull():
    """ Performs mercurial pull """
    cmd = ["hg", "pull"]
    sp.run(cmd)


def git_incoming():
    """ Performs a hg incoming-like function using git own methods"""
    # TODO: use one of the git apis instead of this, but maybe this is the cleanest way to do it?
    cmd = ["git", "fetch"]
    sp.run(cmd)
    cmd = ["git", "log", "..origin/master", "--no-merges", "--format=email"]
    cmd_ran = sp.run(cmd, stdout=sp.PIPE)
    out = cmd_ran.stdout
    changesets = out.decode().split("changeset")[1:]
    msgs = re_git_msg.findall(changesets)
    users = re_git_user.findall(changesets)
    rev_dicts = []
    for msg_raw, user_raw in zip(msgs, users):
        msg = msg_raw.replace("[PATCH]", "").strip()
        user = user_raw.strip()
        branch = ""
        rev_dicts.append({"branch": branch, "user": user, "msg": msg})
    return rev_dicts


def git_pull():
    """ Performs git pull """
    cmd = ["git", "pull", "-f"]
    sp.run(cmd)


def repo_check_incoming(repo_path, max_log_n=4):
    """ Wrapper around git incoming / hg incoming """
    os.chdir(repo_path)
    if os.path.isdir(".hg"):
        commits = mercurial_incoming()
        mercurial_pull()
    elif os.path.isdir(".git"):
        commits = git_incoming()
        git_pull()

    n_commits = len(commits)
    repo_name = os.path.basename(repo_path)
    msg = "{0} new commits found in the {1} repository".format(n_commits, repo_name)
    if n_commits > max_log_n:
        msg += ", showing only the latest {0}".format(max_log_n)
    answer = [msg]
    for commit in commits[:max_log_n]:
        m = "\n > By {0}".format(commit["user"])
        if commit["branch"]:
            m += " in branch {0}".format(commit["branch"])
        m += ": " + commit["msg"]
        answer.append(m)
    return "\n".join(answer)


class Repository(Component):
    """
    Checks in the repository given
        in check_repository for new (i.e., not pulled)
        commits
    """

    def cmdline_command(self, args):
        repository = args.check_repository
        msg = repo_check_incoming(repository)
        self.send_msg(msg)



"""
    Module implementing some functions useful for playing DnD over the internet
"""
import os
import re
import logging
import subprocess as sp
from random import randint
from pybliotecario.components.component_core import Component

log = logging.getLogger(__name__)
# regex to separate +/-
re_pm = re.compile(r"\+|-")
re_mod = re.compile(r"[+-]\d+")

# TODO: write a syntax checker for the diceroll command

def parse_roll(text):
    """
    Gets as input a line of text defining a diceroll command and parses
    it as a list of dice (`dice_list`) a list of signs for each dice (`pm_list`)
    and a list of modifiers (`mod`)
    If the first die has no sign, it adds a '+' sign as default
    """
    # First separate the dice from the modifiers
    modifiers = text.rpartition("d")[-1]
    # Now get all the modifiers (if any)
    nindex = re_pm.search(modifiers)
    if nindex:
        mod = modifiers[nindex.start():]
        dice = text.partition(mod)[0]
    else:
        mod = ""
        dice = text
    dice_list = re_pm.split(dice)
    pm_list_raw = re_pm.findall(dice)
    if len(pm_list_raw) < len(dice_list):
        pm_list_raw.insert(0, '+')
    # Make the signs into 1 or -1
    pm_list = []
    for i in pm_list_raw:
        if i == "+":
            pm_list.append(1)
        elif i == "-":
            pm_list.append(-1)
    return dice_list, pm_list, mod

def parse_dice(text):
    """ Receives a text in the form ndm and returns
    the number of dice and the faces of the die """
    split_dice = text.split('d')
    n_dice = int(split_dice[0])
    n_face = int(split_dice[-1])
    return n_dice, n_face

def roll_dice(text):
    """
    Receives a string containing any number of dice in rol-like format
    (like 1d20+3d6+4) and returns the number as a result
    """
    # Parse the text command to get the dice and the modifiers
    dice_list, pm_list, mod = parse_roll(text)

    result = []
    for dice, sign in zip(dice_list, pm_list):
        n_dice, n_face = parse_dice(dice)
        for i in range(n_dice):
            result.append( sign*randint(1,n_face) )

    # Now sum everything together and add any possible modifiers
    final_result = sum(result)
    final_str = " + ".join( ["({0})".format(i) for i in result] )

    # Evaluate the modifiers with ast
    if mod.strip():
        modifiers = re_mod.findall(mod)
        for i in modifiers:
            final_result += int(i)
        mod = " ".join(modifiers)
    # And now put all together
    final_str += " {0} = {1}".format(mod, final_result)
    return final_str


class DnD(Component):
    section_name = "DnD"
    default_roll = "1d20"

    def telegram_message(self, msg):
        # Get the text and the username
        text = msg.text.strip()
        username = msg.username.strip()
        # Separate possible extra msgs that go with the text
        split_text = text.split(" ",1)
        roll_cmd = split_text[0]
        if len(split_text) == 2:
            roll_text = " " + split_text[1]
        else:
            roll_text = ""
        # Compute the result in the form of a string
        roll_result = roll_dice(roll_cmd)
        # Write the answer
        answer = "{0} rolled{1}:\n{2}".format(username, roll_text, roll_result)
        self.send_msg(answer)



"""
    Wrapper for argument parser and initialization
"""
import os
import glob
import importlib
import configparser
from argparse import ArgumentParser, Action

CONFIG_FILE = "pybliotecario.ini"


def write_config(config_dict, config_file, config_exists=False):
    """
    Receives a dictionary of dictionaries and makes it into a
    ConfigParser
    """
    config = configparser.ConfigParser()
    if config_exists:
        config.read(config_file)
    for key, item in config_dict.items():
        config[key] = item
    with open(config_file, "w") as f:
        config.write(f)


def check_attr(attr):
    """
    Checks whether the attr is a class and whether
    it does inherit from Component and whether
    """
    from pybliotecario.components.component_core import Component

    if not isinstance(attr, type):
        return False
    if attr == Component:
        return False
    if issubclass(attr, Component):
        # Is there a more elegant way of knowing whether a method is overriden?
        return not Component.configure_me.__code__ == attr.configure_me.__code__
    return False


def config_module(module):
    """
    Calls the .configure_me method of all modules which
    inherit from component_core.Component
    """
    actor_list = []
    for name in dir(module):
        name_py = getattr(module, name)
        if check_attr(name_py):
            actor_list.append(name_py)
    dict_list = []
    for Actor in actor_list:
        name = Actor.whoamI()
        yn = input("Do you want to configure {0} now? [yn] ".format(name))
        if not yn.lower().startswith(("y", "s")):
            continue
        result = Actor.configure_me()
        if result:
            dict_list.append(result)
    return dict_list


def configure_telegram(main_folder):
    """ Configure Telegram """
    # Initialize the bot in telegram
    print(
        """Welcome to The Wizard!
The first thing you will need is an authorization token from the botfather.
If you don't know how to get one, read here: https://core.telegram.org/bots#6-botfather"""
    )
    token = input("Authorization token: ")
    print("Thanks, let's test this out. Say something to your bot")
    from pybliotecario.TelegramUtil import TelegramUtil

    teleAPI = TelegramUtil(token, timeout=20)
    while True:
        all_updates = teleAPI.get_updates(not_empty=True)
        from pybliotecario.Message import Message

        update = Message(all_updates[0])
        print("Message received: {0}".format(update.text))
        yn = input("Was this your msg? [y/n] ")
        if yn.lower() in ("y", "s"):
            chat_id = update.chat_id
            print("Your chat id is: {0} and your username is: {1}".format(chat_id, update.username))
            break
        else:
            print("Try again")
    # Fill the DEFAULT options
    config_dict = {"DEFAULT": {"TOKEN": token, "chat_id": chat_id, "main_folder": main_folder}}
    return config_dict


def configure_all():
    """ Import everything inside the components folder that
    inherint from Component and run config_module
    on it """
    config_dict = {}
    # Import everything that inherits from Component
    import pybliotecario.components as components

    folder_components = os.path.dirname(components.__file__)
    module_components = components.__name__
    modules = glob.glob(folder_components + "/*.py")
    for module_file in modules:
        module_name = "{0}.{1}".format(module_components, os.path.basename(module_file))
        module_clean = module_name.replace(".py", "")
        module = importlib.import_module(module_clean)
        dict_list = config_module(module)
        for dictionary in dict_list:
            for key, item in dictionary.items():
                config_dict[key] = item
    return config_dict


class InitAction(Action):
    """
        This class performs the initialization.

        Everything is only imported exactly where it is needed because that way
        also serves a a sort of documentation? I think. It looks instructive...
    """

    def __init__(self, nargs=0, **kwargs):
        super().__init__(nargs=nargs, **kwargs)

    def __call__(self, parser, *args, **kwargs):
        """
            Configures the pybliotecario by first
            configuring Telegram
            and then calling the configure_me method of all components
        """
        config_dict = {}
        # Set up environmental stuff
        home = os.environ["HOME"]
        main_folder = home + "/.pybliotecario/"
        os.makedirs(main_folder, exist_ok=True)
        config_file = home + "/." + CONFIG_FILE
        # Check whether a config file already exists
        config_exists = os.path.isfile(config_file)
        # If it does you might not want to reconfigure Telegram, so let's ask
        initialize = True
        if config_exists:
            print("It seems pybliotecario's Telegram capabilities have already been configured in this computer")
            yn = input("Do you want to configure it again? [y/n] ")
            if not yn.lower().startswith(("y", "s")):
                initialize = False
        if initialize:
            config_dict.update(configure_telegram(main_folder))
        print("Next we will run over the different modules of this program to fill some configuration options")
        config_dict.update(configure_all())
        # And finally write the config file
        write_config(config_dict, config_file, config_exists=config_exists)
        parser.exit(0)


def parse_args():
    """ Wrapper for ArgumentParser """
    parser = ArgumentParser()
    parser.add_argument("message", help="Message to send to Telegram", nargs="*")
    parser.add_argument("--init", help="Wizard to configure the pybliotecario for the first time", action=InitAction)
    parser.add_argument("--config_file", help="Define a custom configuration file")
    parser.add_argument("-d", "--daemon", help="Activate the librarian", action="store_true")
    parser.add_argument("-i", "--image", help="Send iamge to Telegram")
    parser.add_argument("-f", "--file", help="Send file to Telegram")
    parser.add_argument(
        "--arxiv_new", help="Send a msg containing a digest of the new submissions to arxiv", action="store_true"
    )
    parser.add_argument(
        "--weather", help="Sends a msg to telegram with the current weather and a small forecast", action="store_true"
    )
    parser.add_argument(
        "--check_repository",
        help="Sends a msg to telegram with the incoming information for the given repository (receives the base folder of the repository)",
    )
    parser.add_argument(
        "--clear_incoming", help="Clears incoming messages in case something has gone wrong", action="store_true"
    )
    parser.add_argument("--pid", help="Monitor a PID and sends a message when the PID is finished", type=int, nargs="+")
    parser.add_argument("--my_ip", help="Send to the default chat the current IP of the computer", action="store_true")
    parser.add_argument(
        "--debug", help="Write everything to terminal instead of writing to log file", action="store_true"
    )
    return parser.parse_args()



""" Server-helper function to look up the current IP of the program """
import urllib.request
from pybliotecario.components.component_core import Component

import logging

log = logging.getLogger(__name__)


def ip_lookup():
    """ Uses ident.me to find out the current ip of the host """
    external_ip = urllib.request.urlopen("https://ident.me").read().decode("utf8")
    return external_ip


class IpLookup(Component):
    """
        Reads the current IP of the computer and sends it to Telegram
    """

    def telegram_message(self, msg):
        """ If the chat id asking is the correct one
        sends a msg with the current ip, otherwise fails """
        if self.check_identity(msg):
            send_msg = ip_lookup()
        else:
            send_msg = "You are not allowed to see this"
        self.telegram.send_message(send_msg, msg.chat_id)

    def cmdline_command(self, args):
        """ Sends the current ip to self.chatid """
        # First look for the IP
        my_ip = ip_lookup()
        # Then append it to the text and send it to Telegram
        message_text = " ".join(args.message)
        message = "{0} {1}".format(message_text, my_ip)
        self.telegram.send_message(message, self.chat_id)
        log.info("IP: {0}".format(my_ip))
        # Finally, consume the text
        args.message = None



"""
    Module using the arxiv API for prelog.info tracking
    update notifications and quick-querying from the Telegram app
"""

from datetime import datetime
from datetime import timedelta
import time
import os
import arxiv
from pybliotecario.components.component_core import Component
import logging

log = logging.getLogger(__name__)


def is_today(time_struct):
    """
    Checks whether the given time_struct corresponds to today
    or to the previous day
    Remember, the previous day for arxiv purposes only starts at 19.00
    """
    # First we need to find out which day is today
    today = datetime.now()
    wday = today.weekday()  # 0 == Monday
    base_hour = 18

    if wday < 2:
        # If today is monday or tuesday, we should look at thursday/friday for the cutoff
        base_today = today - timedelta(days=4)
    else:  # yesterday
        base_today = today - timedelta(days=2)
    base_today = base_today.replace(hour=base_hour, minute=0, second=0, microsecond=0)

    # Make the time struct into a datetime object
    pdt = datetime.fromtimestamp(time.mktime(time_struct))

    # Now, if the paper date (pdt) is from before the base time (base_today), that means it is not from today
    return pdt >= base_today


def query_recent(category):
    """
    Query the arxiv for the updates of the last day for a given category
    """
    update_key = "updated_parsed"
    results = arxiv.query(query=category, max_results=75, sort_by="lastUpdatedDate")
    indx = -1
    for i, element in enumerate(results):
        time_s = element[update_key]
        if not is_today(time_s):
            indx = i
            break
    return results[:indx]


def filter_results(result_list, filter_dictionary):
    """
    Given a dictionary and a list of results (paper dictionaries)
    returns the results which pass the filter
    The items in the filter_dictionary must be a list
    """  # TODO: save the reason for the query to pass
    lista = []
    for paper in result_list:
        done = False
        for key, item in filter_dictionary.items():
            if done:
                break
            val = paper[key]
            for keyword in item:
                if isinstance(val, str):
                    if keyword.lower() in val.lower():
                        lista.append(paper)
                        done = True
                        break
                elif isinstance(val, list):
                    if done:
                        break
                    for keyval in val:
                        if keyword.lower() in keyval.lower():
                            lista.append(paper)
                            done = True
                            break
    # Ok, this might be very dangerous here
    return lista


def url_to_id(arxiv_url):
    """
    Given an arxiv url return the corresponding id
    if arxiv_url is already an id, returns unchanged
    """
    return os.path.basename(arxiv_url)


# Telegram-usable functions
def arxiv_get_pdf(arxiv_id_raw):
    """ Downloads a paper from the arxiv given an id """
    arxiv_id = url_to_id(arxiv_id_raw)
    # First we recover the information about the paper
    paper = arxiv.query(id_list=[arxiv_id])[0]
    # Now download the pdf (the download command only needs a dictionary with the pdf_url, but that is given by the query)
    file_name = arxiv.download(paper)
    return file_name


def arxiv_recent_filtered(categories, filter_dict, abstract=False):
    """ Read all recent papers in each categories and save the ones matching the
    requirements of filter_dict """
    lines = []
    for category in categories:
        tmp = query_recent(category)
        results = filter_results(tmp, filter_dict)
        line = "{0} new papers in {1}, {2} interesting ones: \n".format(len(tmp), category, len(results))
        for paper in results:
            authors = ", ".join(paper["authors"])
            title = paper["title"]
            arxiv_id = os.path.basename(paper["id"]).replace("v1", "")
            line += " > {2}: {0}\n     by {1}\n".format(title, authors, arxiv_id)
            if abstract:
                line += paper["summary"]
        lines.append(line)
    return "\n".join(lines)


def arxiv_query_info(arxiv_id_raw):
    """
    Returns extra information about the queried paper
    """
    arxiv_id = url_to_id(arxiv_id_raw)
    paper = arxiv.query(id_list=[arxiv_id])[0]
    title = paper["title"]
    authors = ", ".join(paper["authors"])
    abstract = paper["summary"]
    msg = """ > {0}
Title: {1}

Authors: {2}

Abstract: {3}""".format(
        arxiv_id, title, authors, abstract
    )
    return msg


class Arxiv(Component):
    def __init__(self, telegram_object, configuration=None, **kwargs):
        super().__init__(telegram_object, configuration=configuration, **kwargs)
        arxiv_config = self.read_config_section("ARXIV")
        title = arxiv_config.get("title")
        abstract = arxiv_config.get("summary")
        authors = arxiv_config.get("authors")
        self.filter_dict = {
            "title": self.split_list(title),
            "summary": self.split_list(abstract),
            "authors": self.split_list(authors),
        }
        self.categories = self.split_list(arxiv_config["categories"])

    @classmethod
    def configure_me(cls):
        print("")
        print(" # Arxiv module ")
        print("This is the configuration helper for the arxiv module")
        print("First introduce (comma-separated) the categories you are interested in: ")
        categories = input(" > ")
        print("Now introduce (again, comma-separated) the keywords you want to check the titles for")
        title_keywords = input(" > ")
        print("Same for checking the abstract:")
        summary_keywords = input(" > ")
        print("And now authors")
        authors_keywords = input(" > ")
        dict_out = {
            "ARXIV": {
                "categories": categories,
                "title": title_keywords,
                "summary": summary_keywords,
                "authors": authors_keywords,
            }
        }
        return dict_out

    def cmdline_command(self, args):
        """
            Reads the latests arrivals to the arxiv and
            sends a notification if any of the entries fullfill any of
            the requirements set in the config file
        """
        msg = arxiv_recent_filtered(self.categories, self.filter_dict)
        self.send_msg(msg)
        log.info("Arxiv information sent")

    def telegram_message(self, msg):
        command = msg.command
        arxiv_id = msg.text.strip()
        if command in ("arxivget", "arxiv-get"):
            file_send = arxiv_get_pdf(arxiv_id)
            self.send_file(file_send, delete=True)
        else:
            msg = arxiv_query_info(arxiv_id)
            self.send_msg(msg)


if __name__ == "__main__":
    log.info("Testing the arxiv component")
    log.info("Query a category")
    category = "hep-ph"
    recent_papers = query_recent(category)
    dict_search = {"title": ["Higgs"], "authors": ["Cruz-Martinez"], "summary": ["VBF"]}
    filter_papers = filter_results(recent_papers, dict_search)
    tlg_msg = arxiv_recent_filtered([category], dict_search)
    log.info(tlg_msg)

#     log.info("Test download")
#     test_id = "1802.02445"
#     name = arxiv_get_pdf(test_id)
#     os.remove(name)



#!/usr/bin/env python3
import configparser
import sys
import os

from pybliotecario.TelegramUtil import TelegramUtil
from pybliotecario.core_loop import main_loop

# Modify argument_parser.py to read new arguments
from pybliotecario.argument_parser import parse_args, CONFIG_FILE
import pybliotecario.on_cmdline as on_cmdline

import logging

log = logging.getLogger()


def read_config(config_file=None):
    result = None
    config_files = ["{0}/.{1}".format(os.environ.get("HOME"), CONFIG_FILE), CONFIG_FILE]
    if config_file is not None:
        config_files.append(config_file)
    config = configparser.ConfigParser()
    config.read(config_files)
    if config and config.defaults():
        return config
    else:
        print("Before using this program you need to run the --init option in order to configure it")
        sys.exit(-1)


def logger_setup(filename, debug=False):
    """
    Send all logger output to file
    But make sure that errors and warning also go
    to stdout
    """
    # Set the formatter
    formatter = logging.Formatter("[%(levelname)s:%(asctime)s] - %(message)s", "%d/%m/%y %H:%M")
    if debug:
        file_handler = logging.StreamHandler(sys.stdout)
    else:
        # Error handler (not necessary when debug is on):
        error_handler = logging.StreamHandler(sys.stdout)
        error_handler.setFormatter(formatter)
        error_handler.setLevel(logging.WARNING)
        log.addHandler(error_handler)
        # Default handler
        file_handler = logging.FileHandler(filename, "a")
    file_handler.setFormatter(formatter)
    log.addHandler(file_handler)
    log.setLevel(logging.INFO)


def main():
    """ Driver of the pybliotecario """
    args = parse_args()
    config = read_config(args.config_file)
    defaults = config.defaults()
    main_folder = defaults.get("main_folder")
    if not main_folder:
        print("Warning: there is no 'default:main_folder' option set in {0}, using /tmp/".format(args.config_file))
        main_folder = "/tmp/"
    logger_setup(main_folder + "/info.log", debug=args.debug)

    log.info("Initializing the pybliotecario")
    api_token = defaults.get("token")
    if not api_token:
        log.error("No 'default:token' option set in config file, run --init option")
        sys.exit(-1)

    tele_api = TelegramUtil(api_token)

    on_cmdline.run_command(args, tele_api, config)

    if args.daemon:
        log.info("Activating main loop")
        while True:
            main_loop(tele_api, config=config, clear=args.clear_incoming)


if __name__ == "__main__":
    main()



from datetime import datetime
import os
from pybliotecario.Message import Message
import pybliotecario.on_cmd_message as on_cmd_message
import logging

log = logging.getLogger(__name__)


def monthly_folder(base_main_folder):
    main_folder = base_main_folder + "/data"
    ahora = datetime.now()
    y = ahora.year
    m = ahora.strftime("%B")
    folder_name = "{0}/{1}/{2}".format(main_folder, y, m)
    if not os.path.isdir(folder_name):
        os.makedirs(folder_name)
    return folder_name


def write_to_daily_log(main_folder, msg):
    folder = monthly_folder(main_folder)
    day = datetime.now().day
    file_name = "{0}/{1}.log".format(folder, day)
    with open(file_name, "a+") as f:
        f.write(msg)
        f.write("\n")


def still_alive():
    from random import randint

    sentences = [
        "Pong",
        "This was a triumph",
        "HUGE SUCCESS",
        "It's hard to overstate my satisfaction",
        "There's no sense crying over every mistake",
        "You just keep on trying till you run out of cake",
        "I'm being so sincere right now",
        "You tore me to pieces",
        "These points of data make a beautiful line",
        "We're out of beta, we're releasing on time",
        "Think of all the things we learned",
        "Go ahead and leave me",
        "I think I prefer to stay inside",
        "This cake is great, it's so delicious and moist",
        "When I look out there it makes me glad I'm not you",
        "Believe me, I am still alive",
        "I'm doing science and I'm still alive",
        "I feel FANTASTIC and I'm still alive",
        "While you're dying I'll be still alive",
        "When you're dead I will be still alive",
    ]
    r = randint(0, len(sentences) - 1)
    return sentences[r]


def main_loop(teleAPI, config=None, clear=False):
    """
    This function activates a "listener" and waits for updates from Telegram
    No matter what the update is about, we first store the content and then
    if it is a command, we act on the command
    """
    if config:
        accepted_user = config["DEFAULT"]["chat_id"]
    else:
        accepted_user = None
    main_folder = config["DEFAULT"]["main_folder"]
    # Get updates from Telegram
    raw_updates = teleAPI.get_updates(not_empty=True)
    updates = [Message(update) for update in raw_updates]
    # Act on those updates
    for update in updates:
        log.info(update.json)
        try:
            if update.ignore:
                continue
            chat_id = update.chat_id
            if update.isFile:
                # If the update is a file, save the file and we are done
                file_name = update.text.replace(" ", "")
                file_path = "{0}/{1}".format(monthly_folder(), file_name)
                result = teleAPI.download_file(update.fileId, file_path)
                if result:
                    teleAPI.send_message("¡Archivo recibido y guardado!", chat_id)
                    log.info("File saved to {0}".format(file_path))
                else:
                    teleAPI.send_message("There was some problem with this, sorry", chat_id)
                    log.info("Since there was some problem, let's open a pdb console here and you decide what to do")
            elif update.is_command:
                # Calls select command and act on the message
                # the function will receive the whole telegram API so it is allowed to send msgs directly
                # it can choose to send back a response instead
                response = on_cmd_message.act_on_telegram_command(teleAPI, update, config)
                # if response is text, or file, it will be sent to the chat
                if isinstance(response, str):
                    teleAPI.send_message(response, chat_id)
                elif isinstance(response, dict):
                    if response["isfile"]:
                        filepath = response["filename"]
                        teleAPI.send_file(filepath, chat_id)
                        if response["delete"]:
                            os.remove(filepath)
            else:
                # Otherwise just save the msg to the log and send a funny reply
                write_to_daily_log(main_folder, update.text)
                random_msg = still_alive()
                teleAPI.send_message(random_msg, chat_id)
        except Exception as e:
            # If we are in clear mode, we want to recapture updates to ensure we clear the ones that produce a fail
            # in principle in clear mode we don't care what the fail is about, we just want to clear the failure
            if clear:
                log.info("\n     > > This update produced an exception: {0}\n\n".format(e))
            else:
                raise e



import pathlib
from setuptools import setup, find_packages
from sys import argv

if len(argv) > 1 and argv[1] in ('develop', 'install'):
    import socket
# When installing with setup.py in develop/install mode
# use hostname as the name of the script
# with the first letter uppercased
    hostname = socket.gethostname()
    pybliotecario_name = hostname.capitalize()
else:
    pybliotecario_name = "pybliotecario"

# Readup the readme
README = (pathlib.Path(__file__).parent / "readme.md").read_text()
setup(
    name=pybliotecario_name,
    version="1.0",

    author="Scarlehoff",
    author_email="juacrumar@lairen.eu",
    url="https://github.com/scarlehoff/pybliotecario",

    description="Personal telegram bot to interact between your Telegram account and your computer",
    long_description=README,
    long_description_content_type="text/markdown",
    license="GNU GPLv3",

    package_dir = {'':'src'},
    packages=find_packages('src'),

    install_requires=[
        'arxiv',
        'pyowm',
        'psutil',
    ],

    entry_points = {'console_scripts':
                    ['{0} = pybliotecario.pybliotecario:main'.format(pybliotecario_name),]
                    },
)

print("""

##############
Installed pybliotecario as {0}

IMPORTANT: Don't forget to run
      ~$ {0} --init
for proper configuration of the pybliotecario""".format(pybliotecario_name))



###### Esta clase tiene que ser redefinida de forma uqe definamos un MessageParser con una serie de opciones y un Message
###### donde el message parser es algo en plan que define una serie de opciones y el message pues tiene todos los getters


registeredCommands = []
import logging

log = logging.getLogger(__name__)


class Message:
    # Variables that we are going to parse from json:
    # chat_id              - Id of the chat the message came from
    # username            - user who sent the message
    # is_command           - t/f
    # isRegisteredCommand - t/f
    # has_arguments       - t/f
    # is_group             - t/f
    # isFile              - t/f
    # fileId              - file id
    # text                - actual content of the message (minus /command)
    # command             - command given in /command (or "")
    # ignore              - t/f (whether this message should be ignored)

    def __init__(self, jsonDict):
        # ignore keys:
        ign_keys = ["new_chat_participant", "left_chat_participant", "sticker"]
        msg = "message"
        self.json = jsonDict
        keys = jsonDict.keys()
        if msg not in keys:
            if "edited_message" in keys:
                msg = "edited_message"
            elif "edited_channel_post" in keys:
                msg = "edited_channel_post"
        try:
            message = jsonDict[msg]
        except:
            log.info("   >>>>> ")
            log.info(jsonDict)
            raise Exception("Not a message or an edited message?")
        msgKeys = message.keys()
        if set(ign_keys) & set(msgKeys):
            self.ignore = True
            return
        else:
            self.ignore = False
        self.has_arguments = False
        chatData = message["chat"]
        if "from" in message.keys():
            fromData = message["from"]
        else:
            fromData = chatData  # something has changed or was this a special type of msg???
        # Populate general fields
        # Check whetehr username exists, otherwise use name, otherwise, unknown
        if "username" in fromData:
            self.username = fromData["username"]
        elif "first_name" in fromData:
            self.username = fromData["first_name"]
        elif "last_name" in fromData:
            self.username = fromData["last_name"]
        else:
            self.username = "unknown_user"
        self.chat_id = chatData["id"]

        # Check the filetyp of what we just received
        if "photo" in msgKeys:
            self.isFile = True
            photoData = message["photo"][-1]
            self.fileId = photoData["file_id"]
            if "caption" in msgKeys:
                self.text = message["caption"]
            else:
                self.text = "untitled"
            if not self.text.endswith((".jpg", ".JPG", ".png", ".PNG")):
                self.text += ".jpg"
        elif "document" in msgKeys:
            self.isFile = True
            fileData = message["document"]
            self.fileId = fileData["file_id"]
            self.text = fileData["file_name"]
        elif "sticker" in msgKeys:
            self.isSticker = True
            stickerData = message["sticker"]
            self.stickerSet = stickerData["set_name"]
        else:
            self.text = message.get("text")
            self.isFile = False

        # Check whether the msg comes from a group
        self.is_group = chatData["type"] is "group"

        #  Now check whether the msg has the structure of a command
        if self.text[0] == "/":
            self.is_command = True
        else:
            self.is_command = False
            self.command = ""
            self.isRegisteredCommand = False

        if self.is_command:
            all_text = self.text.split(" ", 1)
            # Check whether the command comes by itself or has arguments
            if len(all_text) > 1:
                self.has_arguments = True
            # Remove the /
            self.command = all_text[0][1:]
            # Absorb the @ in case is it a directed command
            if "@" in self.command:
                self.command = self.command.split("@")[0]
            self.text = all_text[-1]
            self.isRegisteredCommand = self.command in registeredCommands



"""
    This module uses the OpenWeather API to provide
    weather information for given location/given times
    using: https://github.com/csparpa/pyowm
"""

from datetime import datetime
from types import MethodType
import logging
from pyowm import OWM
from pybliotecario.components.component_core import Component

log = logging.getLogger(__name__)


def check_current_weather(self, location):
    """ Checks the current weather at a specific location """
    obs = self.weather_at_place(location)
    weather = obs.get_weather()

    temp = weather.get_temperature(unit="celsius")["temp"]
    stat = weather.get_detailed_status()
    city = location.split(",")[0]

    msg = "Right now we have {0} in {2}, the current temperature is {1}°C"
    return msg.format(stat, temp, city)


def get_forecast(self, location):
    """ Gets the full forecast for a given location """
    forecast = self.three_hours_forecast(location)
    return forecast
    # f = fc.get_forecast()


# Telegram-usable wrapper
def will_it_rain(self, location, hours=None):
    """
    Checks the forecast for rain today at the given times
    """
    if hours is None:
        hours = ["10", "17"]
    forecast = self.get_forecast(location)
    rains = []
    for hour in hours:
        if check_for_rain_today_at(forecast, hour):
            rains.append(hour)
    if rains:
        rain_times = ", ".join(rains)
        msg = "I find rain at the following times: {0} ".format(rain_times)
    else:
        msg = "I can see no rain in my foreseeable future"
    return msg


def check_for_rain_today_at(forecast, hour):
    """
    Receives a forecast object
    Returns true if rain is found at the given hour
    """
    now = datetime.now()
    check = now.replace(hour=int(hour))
    if check > now and forecast.will_be_rainy_at(check):
        return True
    else:
        return False


def open_weather_wrapper(api_key, **kwargs):
    """
        Extends OpenWeather main class to have an instance with the API key already defined
    """
    owm = OWM(api_key, **kwargs)
    owm.check_current_weather = MethodType(check_current_weather, owm)
    owm.get_forecast = MethodType(get_forecast, owm)
    owm.will_it_rain = MethodType(will_it_rain, owm)
    return owm


class Weather(Component):
    """
        Checks the weather forecast at a given location
        at some given times as well as the current forecast at the
        given location and reports to Telegram the findings
    """

    def __init__(self, telegram_object, configuration=None, **kwargs):
        super().__init__(telegram_object, configuration=configuration, **kwargs)
        weather_config = self.read_config_section("WEATHER")
        self.api_key = weather_config.get("api")
        self.weather_location = weather_config.get("location")
        times_str = weather_config.get("times")
        self.check_times = self.split_list(times_str)

    @classmethod
    def configure_me(cls):
        print("")
        print(" # WEATHER MODULE # ")
        print("In order to configure the weather module, you need an API token from OpenWeatherMap")
        print("Go here in order to get one: https://openweathermap.org/appid")
        print("Introduce below the API token, leave empty to skip the configuration of the weather module")
        api = input(" > ")
        if api.strip() == "":
            return None
        print("At which times (comma-separated) do you want to check the weather?")
        times = input(" > ")
        print("In which city?")
        location = input(" > ")
        dict_out = {"WEATHER": {"api": api, "location": location, "times": times}}
        return dict_out

    def cmdline_command(self, args):
        # instantiate open wather object
        open_weather = open_weather_wrapper(self.api_key)
        # ask whether it will rain in the future
        msg_rain = open_weather.will_it_rain(self.weather_location, self.check_times)
        # ask what is the current weather
        msg_current = open_weather.check_current_weather(self.weather_location)
        # join everything on a nice msg and send
        weather_msg = "{1}\n{0}".format(msg_rain, msg_current)
        self.send_msg(weather_msg)
        log.info("Weather information sent")


if __name__ == "__main__":
    log.info("Testing weather")
    weather_api = "<weather api>"
    city = "Milano, IT"
    msg = will_it_rain(city, weather_api)
    log.info("Check of the forecast")
    log.info(msg)
    log.info("Check current weather")
    msg = check_current_weather(city, weather_api)
    log.info(msg)
